def print_menu():
    print('''
    yfinance2pg
        Download financial data to postgres database from yahoo finance.

    usage:
        yfinance2pg <options>

    options:
        --help
            help menu
        --exclude [value ...]
            skip download phase, possible list
            values are `companies` or `priceVolume`
        --host [string]
        --user [string]
        --dbname [string]
        --port [string]
        --password [string]
            postgres connection options
    ''')



import psycopg2
import psycopg2.extras
import os

from .helpers import get_measurement_type
from .config import config


def connect(**kwargs):
    try:
        return psycopg2.connect(**kwargs)
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)

    try:
        return connect_default()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)


def connect_default():
    try:
        params = config(section='postgresql')
        return psycopg2.connect(**params)
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)


def init(curs):
    file = open(os.path.join('schema.sql'), 'r')
    schema = file.read()
    curs.execute(schema)


def insert_symbols(curs, symbols):
    insert_query = '''
        INSERT INTO Company
        (Ticker, Name, Exchange, Industry, Sector)
        VALUES %s ON CONFLICT DO NOTHING
    '''

    psycopg2.extras.execute_values(
        curs, insert_query, symbols, template=None, page_size=100
    )


def insert_price_volume_measurement(curs, date, symbol, measure_label, value):
    m_type = get_measurement_type(measure_label)
    format_args = (m_type,) * 3
    insert_query = '''
        INSERT INTO PriceVolume
        (Ticker, Day, {})
        VALUES (%s, %s, %s)
        ON CONFLICT (Ticker, Day) DO UPDATE
        SET Ticker=EXCLUDED.Ticker,
        Day=EXCLUDED.Day,
        {}=EXCLUDED.{}
    '''.format(*format_args)

    values = (symbol, date, value,)
    curs.execute(insert_query, values)


def get_symbols(curs):
    curs.execute('SELECT Ticker FROM Company')

    fetched = curs.fetchall()
    all_symbols = map(lambda x: x[0], fetched)
    return list(all_symbols)


def get_from_date(curs):
    curs.execute('SELECT MAX(Day) FROM PriceVolume')
    return str(curs.fetchone()[0] or '1970-01-01')



import sys


def get(arg):
    try:
        idx = sys.argv.index('--' + arg)
        return sys.argv[idx + 1]
    except Exception:
        return None


def get_list(arg):
    try:
        idx = sys.argv.index('--' + arg)
        return sys.argv[idx + 1].split(',')
    except Exception:
        return []


def exists(arg):
    try:
        sys.argv.index('--' + arg)
        return True
    except Exception:
        return False



import traceback
import time
import datetime
import math

from .db import connect, init
from .download import companies, price_volume
from .help_text import print_menu
from .args import exists, get_list, get

if exists('help'):
    print_menu()
    exit(0)

exclude = get_list('exclude')
download_companies = 'companies' not in exclude
download_price_volume = 'priceVolume' not in exclude
start_date = get('start-date')

connection_options = {
    'host': get('host'),
    'password': get('password'),
    'user': get('user'),
    'dbname': get('dbname'),
    'port': get('port')
}


def main():
    start = time.time()

    try:
        conn = connect(**connection_options)
        curs = conn.cursor()

        init(curs)
        conn.commit()

        if download_companies:
            companies(curs, conn.commit)

        if download_price_volume:
            price_volume(curs, start_date, conn.commit)

        curs.close()

        end = time.time()
        total = math.floor(end - start)
        total_time = str(datetime.timedelta(seconds=total))
        print('finished {0} h:mm:ss'.format(total_time))
    except KeyboardInterrupt:
        pass
    except Exception:
        traceback.print_exc()
    finally:
        if conn is not None:
            conn.close()



def chunks(lst, n):
    for i in range(0, len(lst), n):
        yield lst[i:i + n]


def get_measurement_type(measure):
    if measure == 'Adj Close':
        return 'AdjustedClosePrice'
    if measure == 'Volume':
        return measure
    else:
        return measure + 'Price'



from configparser import ConfigParser


def config(filename='database.ini', section='postgresql'):
    parser = ConfigParser()
    parser.read(filename)

    results = {}
    if parser.has_section(section):
        params = parser.items(section)
        for param in params:
            results[param[0]] = param[1]
    else:
        raise Exception(
            'Section {0} not found in the {1} file'.format(section, filename)
        )

    return results



import setuptools

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name='yfinance2pg',  # Replace with your own username
    version='1.0.1',
    author='Regan Karlewicz',
    author_email='yfinance2pg@karlewr.net',
    description='Download finance data from yfinance to postgres database',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/regan-karlewicz/yfinance2pg',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    entry_points='''
        [console_scripts]
        yfinance2pg=yfinance2pg.main:main
    ''',
    install_requires=[
        'yfinance>=0.1.55',
        'psycopg2>=2.8.6',
        'requests>=2.25.1',
        'html5lib>=1.1',
        'bs4>=0.0.1',
        'multitasking>=0.0.9'
    ],
    python_requires='>=3.6',
)



import datetime
import yfinance
import math
import multitasking
import pandas as pd

from .tickers import get_symbols
from .helpers import chunks
from .db import \
    get_symbols as get_downloaded, \
    get_from_date, \
    insert_price_volume_measurement, \
    insert_symbols


@multitasking.task
def download_company(symbol, companies):
    try:
        try:
            data = yfinance.Ticker(symbol).info
            industry = data.get('industry')
            sector = data.get('sector')
            name = data.get('longName')
        except Exception:
            industry = None
            sector = None
            name = None
        companies.append((symbol, name, 'US', industry, sector,))
    except Exception as e:
        print('Failed for {0}: {1}'.format(symbol, e))


def companies(curs, commit):
    print('== Downloading company data ==')

    symbols = get_symbols()
    downloaded = set(get_downloaded(curs))

    companies = []

    current_chunk = 0
    chunk_size = 100
    total_chunks = math.ceil(len(symbols) / chunk_size)

    for symbol_chunk in chunks(symbols, chunk_size):
        current_chunk += 1
        print(
            'Downloading chunk {0} of {1}'
            .format(current_chunk, total_chunks)
        )

        any_downloaded = False
        for symbol in symbol_chunk:
            if symbol in downloaded:
                continue

            any_downloaded = True
            download_company(symbol, companies)

        if any_downloaded:
            multitasking.wait_for_tasks()

    if len(companies) > 0:
        insert_symbols(curs, companies)
        commit()


def price_volume(curs, start_date, commit):
    print('== Downloading price volume data ==')

    symbols = get_symbols()
    start = start_date or get_from_date(curs)
    end = str(datetime.date.today())

    print('Start date: ', start)

    current_chunk = 0
    chunk_size = 100
    total_chunks = math.ceil(len(symbols) / chunk_size)

    for symbol_chunk in chunks(symbols, chunk_size):
        current_chunk += 1
        print('Downloading chunk {0} of {1}'.format(
            current_chunk, total_chunks
        ))

        data = yfinance.download(
            ' '.join(symbol_chunk),
            start=start,
            end=end
        )

        for day in data.iterrows():
            date = day[0]
            prices = day[1]

            for index, value in prices.items():
                measure, symbol = index

                if pd.isna(value):
                    continue

                insert_price_volume_measurement(
                    curs, date, symbol, measure, value
                )

            commit()



import requests

endpoint = 'https://www.sec.gov/include/ticker.txt'


def get_symbols():
    tickers = []

    r = requests.get(endpoint)

    for line in r.text.split('\n'):
        tickers.append(line.split('\t')[0].upper())

    return tickers
